#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
@File Name  : __init__.py
@Author     : LeeCQ
@Date-Time  : 2021/1/8 19:55

模块对应关系：
1. DBBase, APIBase
2. BaseSQL(DBBase)  ,  NoSQLBase(DBBase)
3.1 MyBaseSQL(BaseSQL, APIBase)  ,  SQLiteBASE(BaseSQL, APIBase)
3.2 MongdbBASE(NoSQLBase, APIBase)  ,

"""
